@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2001 Chris Lightfoot. All rights reserved.
  *
- * $Id: auth_perl.h,v 1.2 2001/04/26 23:30:03 chris Exp $
+ * $Id: auth_perl.h,v 1.7 2002/06/25 20:28:00 chris Exp $
  *
  */
 
@@ -23,9 +23,11 @@
 void xs_init(void);
 int auth_perl_init(void);
 void auth_perl_close(void);
+void auth_perl_postfork(void);
 stringmap auth_perl_callfn(const char *perlfn, const int nvars, ...);
-authcontext auth_perl_new_apop(const char *name, const char *timestamp, const unsigned char *digest);
-authcontext auth_perl_new_user_pass(const char *user, const char *pass);
+authcontext auth_perl_new_apop(const char *name, const char *local_part, const char *domain, const char *timestamp, const unsigned char *digest, const char *clienthost, const char *serverhost);
+authcontext auth_perl_new_user_pass(const char *user, const char *local_part, const char *domain, const char *pass, const char *clienthost, const char *serverhost);
+void auth_perl_onlogin(const authcontext A, const char *clienthost, const char *serverhost);
 
 #endif /* AUTH_PERL */
 
