@@ -9,7 +9,7 @@
  *
  */
 
-static const char rcsid[] = "$Id: cfgdirectives.c,v 1.24 2003/08/25 23:54:27 chris Exp $";
+static const char rcsid[] = "$Id: cfgdirectives.c,v 1.26 2003/11/24 19:58:28 chris Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -55,14 +55,19 @@ char *cfgdirectives[] = {
 #endif
 
 #ifdef USE_TLS
-    "tls-bug-workarounds",
+    "tls-no-bug-workarounds",
 #endif
 
+    "authcache-enable",
+    "authcache-use-client-host",
+    "authcache-entry-lifetime",
+
 #ifdef AUTH_PAM
     /* auth-pam options */
     "auth-pam-enable",
     "auth-pam-facility",
     "auth-pam-mailbox",
+    "auth-pam-mail-user",
     "auth-pam-mail-group",
 #endif
 
