@@ -6,7 +6,7 @@
  *
  */
 
-static const char rcsid[] = "$Id: connection.c,v 1.49 2003/07/14 23:31:20 chris Exp $";
+static const char rcsid[] = "$Id: connection.c,v 1.52 2003/11/06 01:19:27 chris Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -381,9 +381,30 @@ void pop3command_delete(pop3command p) {
  * Send a +OK... / -ERR... response to a message. Returns 1 on success or 0 on
  * failure. */
 int connection_sendresponse(connection c, const int success, const char *s) {
-    if (connection_send(c, success ? "+OK " : "-ERR ", success ? 4 : 5)
-            && connection_send(c, s, strlen(s))
-            && connection_send(c, "\r\n", 2)) {
+    /*
+     * For efficiency's sake, we should send this bit-by-bit, avoiding another
+     * buffer copy. But unfortunately, there are POP3 clients in the world
+     * so stupid that they assume a whole response will arrive in a single TCP
+     * segment. Particular examples include POP3 virus-scanning proxies, such
+     * as Norman ASA's, which was evidently written by somebody very lazy.
+     * 
+     * Obviously there's no way to guarantee how the packets in a TCP stream
+     * are disposed, in general, but we can increase the probability of success
+     * by trying to ensure here that our response is contained in a single
+     * write call. It still might get split up by the ioabs layer, but we have
+     * to take our chances....
+     */
+    static char *buf;
+    static size_t buflen;
+    size_t l;
+    
+    l = (success ? 6 : 7) + strlen(s);
+
+    if (!buf || buflen < l + 1)
+        buf = xrealloc(buf, buflen = l + 1);
+    
+    sprintf(buf, "%s %s\r\n", success ? "+OK" : "-ERR", s);
+    if (connection_send(c, buf, l)) {
         if (verbose)
             log_print(LOG_DEBUG, _("connection_sendresponse: client %s: sent `%s %s'"), c->idstr, success ? "+OK" : "-ERR", s);
         return 1;
@@ -406,11 +427,11 @@ int connection_sendline(connection c, const char *s) {
  * terminated with `\r\n'. If n is -1, the whole message is sent.
  *
  * RFC1939 doesn't define what a server which encounters an error half-way
- * through sending a message. In any case it's clear that we mustn't send the
- * final ., since that would result in the user obtaining a truncated message.
- * So we return -1 if the message could not be sent but a -ERR response was
- * transmitted to the client, -2 if sending failed after a +OK response was
- * sent, or the number of bytes written on success.
+ * through sending a message should do. In any case it's clear that we mustn't
+ * send the final ., since that would result in the user obtaining a truncated
+ * message.  So we return -1 if the message could not be sent but a -ERR
+ * response was transmitted to the client, -2 if sending failed after a +OK
+ * response was sent, or the number of bytes written on success.
  *
  * Assumes the message on disk uses only `\n' to indicate EOL. */
 int connection_sendmessage(connection c, int fd, size_t msgoffset, size_t skip, size_t msglength, int n) {
@@ -438,7 +459,7 @@ int connection_sendmessage(connection c, int fd, size_t msgoffset, size_t skip,
     p += skip;
 
     /* Send the message headers */
-	 while (p < r && *p != '\n') {
+    while (p < r && *p != '\n') {
         q = memchr(p, '\n', r - p);
         if (!q) q = r;
         errno = 0;
