@@ -6,7 +6,7 @@
  * comedy attempt at cclient locking, which is only there so that PINE figures
  * out when the user is attempting to pick up her mail using POP3 in the
  * middle of a PINE session. cclient locks aren't made, just stolen from PINE
- * using the wacky "Kiss Of Death" described in the cclient documentation.
+ * using the wacky `Kiss Of Death' described in the cclient documentation.
  *
  * Note also that we lock the whole mailspool for reading and writing. This is
  * pretty crap, but it makes it easier to make the program fast. In principle,
@@ -18,7 +18,7 @@
  *
  */
 
-static const char rcsid[] = "$Id: locks.c,v 1.7 2001/11/07 19:25:41 chris Exp $";
+static const char rcsid[] = "$Id: locks.c,v 1.10 2003/01/24 11:31:24 chris Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -50,8 +50,7 @@ static const char rcsid[] = "$Id: locks.c,v 1.7 2001/11/07 19:25:41 chris Exp $"
 #ifdef WITH_FCNTL_LOCKING
 /* fcntl_lock:
  * Attempt to lock a file using fcntl(2) locking. Returns 0 or success, or -1
- * on error.
- */
+ * on error. */
 int fcntl_lock(int fd) {
     struct flock fl = {0};
 
@@ -66,8 +65,7 @@ int fcntl_lock(int fd) {
 
 /* fcntl_unlock:
  * Attempt to unlock a file using fcntl(2) locking. Returns 0 on success, or
- * -1 on error.
- */
+ * -1 on error. */
 int fcntl_unlock(int fd) {
     struct flock fl = {0};
 
@@ -84,26 +82,48 @@ int fcntl_unlock(int fd) {
 #if defined(WITH_FLOCK_LOCKING) || (defined(WITH_CCLIENT_LOCKING) && !defined(CCLIENT_USES_FCNTL))
 /* flock_lock:
  * Attempt to lock a file using flock(2) locking. Returns 0 on success, or -1
- * on failure.
- */
+ * on failure. */
 int flock_lock(int fd) {
     return flock(fd, LOCK_EX | LOCK_NB);
 }
 
 /* flock_unlock:
  * Attempt to unlock a file using flock(2) locking. Returns 0 on success or -1
- * on failure.
- */
+ * on failure. */
 int flock_unlock(int fd) {
     return flock(fd, LOCK_UN);
 }
 #endif /* WITH_FLOCK_LOCKING */
 
 #ifdef WITH_DOTFILE_LOCKING
+/* dotfile_check_stale:
+ * If the named lockfile exists, then check whether the PID inside it is one
+ * for an extant process. If it is not, remove it. */
+static void dotfile_check_stale(const char *file) {
+    int fd;
+    char buf[16] = {0};
+    fd = open(file, O_RDONLY);
+    if (fd == -1) return;
+    if (read(fd, buf, sizeof(buf) - 1) > 0) {
+        int i;
+        i = strspn(buf, "0123456789");
+        if (i > 0 && buf[i] == '\n') {
+            pid_t p;
+            /* Have a valid PID, possibly. */
+            buf[i] = 0;
+            p = (pid_t)atoi(buf);
+            if (p > 1 && kill(p, 0) == -1 && errno == ESRCH
+                && unlink(file) == 0)
+                /* File exists but process doesn't. */
+                log_print(LOG_INFO, _("dotfile_check_stale: removed stale lockfile `%s' (pid was %d)"), file, (int)p);
+        }
+    }
+    close(fd);
+}
+
 /* dotfile_lock:
  * Attempt to lock a file by constructing a lockfile having the name of the
- * file with ".lock" appended. Returns 0 on success or -1 on failure.
- */
+ * file with ".lock" appended. Returns 0 on success or -1 on failure. */
 int dotfile_lock(const char *name) {
     char *lockfile = xmalloc(strlen(name) + 6), *hitchfile = NULL;
     char pidstr[16];
@@ -117,6 +137,8 @@ int dotfile_lock(const char *name) {
     if (!lockfile) goto fail;
     sprintf(lockfile, "%s.lock", name);
 
+    dotfile_check_stale(lockfile);
+
     /* Make a name for a hitching-post file. */
     if (uname(&uts) == -1) goto fail;
     hitchfile = xmalloc(strlen(name) + strlen(uts.nodename) + 24);
@@ -141,8 +163,7 @@ int dotfile_lock(const char *name) {
     unlink(hitchfile);
 
     /* Were we able to link the hitching post to the lockfile, and if we were,
-     * did it have exactly 2 links when we were done?
-     */
+     * did it have exactly 2 links when we were done? */
     if (rc != 0 && st.st_nlink != 2) {
         log_print(LOG_ERR, _("dotfile_lock(%s): unable to link hitching post to lock file: %m"), name);
         goto fail;
@@ -162,8 +183,7 @@ fail:
  * Unlock a file which has been locked using dotfile locking. Returns 0 on
  * success or -1 on failure.
  *
- * XXX We try to check that this is _our_ lockfile. Is this correct?
- */
+ * XXX We try to check that this is _our_ lockfile. Is this correct? */
 int dotfile_unlock(const char *name) {
     char pidstr[16], pidstr2[16] = {0};
     char *lockfile = xmalloc(strlen(name) + 6);
@@ -211,8 +231,7 @@ fail:
 /* cclient_steal_lock:
  * Attempt to steal a c-client lock (if any) applied to the file. Returns 0 on
  * success, or -1 on failure. This is fairly comedy, but it is good enough to
- * get PINE to get out of the way when necessary.
- */
+ * get PINE to get out of the way when necessary. */
 int cclient_steal_lock(int fd) {
     struct stat st;
     char cclient_lockfile[64], other_pid[128] = {0};
@@ -227,8 +246,7 @@ int cclient_steal_lock(int fd) {
      *
      * XXX exim lstats the /tmp/... file to ensure that it is not a symbolic
      * link. Since we don't actually write to the file, it is probably not
-     * necessary to make this check.
-     */
+     * necessary to make this check. */
     fd_cc = open(cclient_lockfile, O_RDWR);
     if (fd_cc == -1) {
         if (errno == ENOENT) /* File did not exist; this is OK. */
@@ -240,8 +258,7 @@ int cclient_steal_lock(int fd) {
 
     /* On most systems, the c-client library uses flock(2) to lock files. Some
      * systems do not have flock(2) (Solaris <cough>), or patch PINE to use
-     * fcntl(2) locking (RedHat <cough>).
-     */
+     * fcntl(2) locking (RedHat <cough>). */
 #ifdef CCLIENT_USES_FCNTL
     if (fcntl_lock(fd_cc) == -1) {
 #else
