@@ -7,7 +7,7 @@
  */
 
 static const char copyright[] = "$Copyright: (c) 2001 Chris Lightfoot. All rights reserved. $";
-static const char rcsid[] = "$Id: main.c,v 1.91 2003/09/07 15:41:24 chris Exp $";
+static const char rcsid[] = "$Id: main.c,v 1.93 2003/11/24 19:58:28 chris Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -416,6 +416,9 @@ int main(int argc, char **argv, char **envp) {
     /* Start logging. */
     log_init();
 
+    /* Maybe start up authentication cache. */
+    authcache_init();
+
     /* Perhaps we have been asked to save metadata caches for BSD mailspools? */
 #if defined(MBOX_BSD) && defined(MBOX_BSD_SAVE_INDICES)
     if (config_get_string("mailspool-index")) {
@@ -494,7 +497,7 @@ retry_pid_file:
             break;
 
         default:
-            max_running_children = 16;
+            max_running_children = 100;
     }
 
     /* Should we automatically append or strip domain names and retry authentication? */
@@ -545,8 +548,10 @@ retry_pid_file:
    
     net_loop();
 
-    if (!post_fork)
+    if (!post_fork) {
         authswitch_close();
+        authcache_close();
+    }
 
     if (listeners) {
         item *I;
