@@ -28,10 +28,13 @@
 #include <string.h>
 #include <syslog.h>
 
+#define _XOPEN_SRC      /* crypt(3), on some systems */
+#include <unistd.h>
+
 #include "md5.h"
 #include "util.h"
 
-static const char rcsid[] = "$Id: password.c,v 1.5 2003/09/07 15:41:24 chris Exp $";
+static const char rcsid[] = "$Id: password.c,v 1.6 2003/09/09 22:52:30 chris Exp $";
 
 /* 
  * MD5 crypt(3) routines. This is here so that you can migrate passwords from
