@@ -6,7 +6,7 @@
  *
  */
 
-static const char rcsid[] = "$Id: pop3.c,v 1.54 2003/08/25 23:54:27 chris Exp $";
+static const char rcsid[] = "$Id: pop3.c,v 1.57 2003/11/06 01:19:27 chris Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -31,7 +31,8 @@ extern int verbose;
 
 int append_domain;  /* Do we automatically try user@domain if user alone fails to authenticate? */
 int strip_domain;   /* Automatically try user if user@domain fails? */
-int apop_only;      /* Disconnect any client which says USER. */
+int apop_only;      /* Disconnect any client which says USER, unless their
+                       connection is secured. */
 int log_bad_pass;   /* Log bad passwords for support purposes. */
 
 /* do_capa CONNECTION COMMAND
@@ -50,7 +51,7 @@ static enum connection_action do_capa(connection c) {
 
     connection_sendresponse(c, 1, _("Capability list follows"));
     for (p = capas; *p; ++p) {
-        if (strcmp(*p, "USER") == 0 && apop_only)
+        if (strcmp(*p, "USER") == 0 && (apop_only && !c->secured))
             continue;
         
 #ifdef USE_TLS
@@ -60,6 +61,7 @@ static enum connection_action do_capa(connection c) {
 
         connection_sendline(c, *p);
     }
+
     connection_sendline(c, ".");
     return do_nothing;
 }
@@ -376,7 +378,7 @@ enum connection_action connection_do(connection c, const pop3command p) {
                 return do_capa(c);
             
             case USER:
-                if (apop_only) {
+                if (apop_only && !c->secured) {
                     connection_sendresponse(c, 0, _("Sorry, you must use APOP"));
                     return close_connection;
                 } else if (!do_user(c, p))
@@ -384,7 +386,7 @@ enum connection_action connection_do(connection c, const pop3command p) {
                 break;
 
             case PASS:
-                if (apop_only) {
+                if (apop_only && !c->secured) {
                     connection_sendresponse(c, 0, _("Sorry, you must use APOP"));
                     return close_connection;
                 } else if (!do_pass(c, p))
