@@ -6,7 +6,7 @@
  *
  */
 
-static const char rcsid[] = "$Id: signals.c,v 1.16 2002/03/18 23:47:34 chris Exp $";
+static const char rcsid[] = "$Id: signals.c,v 1.23 2003/07/18 08:26:00 chris Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -65,33 +65,24 @@ void set_signals() {
         0};
     int terminate_signals[] = {SIGINT, SIGTERM, 0};
     int restart_signals[]   = {SIGHUP, 0};
-    int die_signals[]       = {SIGQUIT, SIGABRT, SIGSEGV, SIGBUS, 0};
+    int die_signals[]       = {SIGQUIT, SIGABRT, SIGSEGV, SIGBUS, SIGILL, 0};
     int *i;
-    struct sigaction sa, saz = {0};
+    struct sigaction sa = {0};
 
-    for (i = ignore_signals; *i; ++i) {
-        sa = saz;
-        sa.sa_handler = SIG_IGN;
-        sigaction(*i, &sa, NULL);
-    }
-
-    for (i = terminate_signals; *i; ++i) {
-        sa = saz;
-        sa.sa_handler = terminate_signal_handler;
-        sigaction(*i, &sa, NULL);
-    }
+    sigemptyset(&sa.sa_mask);
+    sa.sa_flags = SA_RESTART;
     
-    for (i = restart_signals; *i; ++i) {
-        sa = saz;
-        sa.sa_handler = restart_signal_handler;
-        sigaction(*i, &sa, NULL);
-    }
+    for (i = ignore_signals; *i; ++i)
+        xsignal(*i, SIG_IGN);
+    
+    for (i = terminate_signals; *i; ++i)
+        xsignal(*i, terminate_signal_handler);
+    
+    for (i = restart_signals; *i; ++i)
+        xsignal(*i, restart_signal_handler);
 
-    for (i = die_signals; *i; ++i) {
-        sa = saz;
-        sa.sa_handler = die_signal_handler;
-        sigaction(*i, &sa, NULL);
-    }
+    for (i = die_signals; *i; ++i)
+        xsignal(*i, die_signal_handler);
 
     /* SIGCHLD is special. */
     sa.sa_handler = child_signal_handler;
@@ -99,10 +90,10 @@ void set_signals() {
     sigaction(SIGCHLD, &sa, NULL);
 }
 
+extern sig_atomic_t foad, restart;       /* in netloop.c */
+
 /* terminate_signal_handler:
  * Signal handler to handle orderly termination of the program. */
-extern int foad;                            /* in main.c */
-
 void terminate_signal_handler(const int i) {
     foad = i;
 }
@@ -116,11 +107,11 @@ void terminate_signal_handler(const int i) {
  * be undefined behaviour too. */
 extern connection this_child_connection;    /* in main.c */
 
-extern char * pidfile;    /* in main.c */
+extern char *pidfile;    /* in main.c */
 extern int post_fork;    /* in main.c */
 
 void die_signal_handler(const int i) {
-    struct sigaction sa;
+    struct sigaction sa = {0};
 /*    log_print(LOG_ERR, "quit: %s", sys_siglist[i]); */
     log_print(LOG_ERR, _("quit: signal %d post_fork = %d"), i, post_fork); /* Some systems do not have sys_siglist. */
 #ifdef APPALLING_BACKTRACE_HACK
@@ -128,8 +119,7 @@ void die_signal_handler(const int i) {
 #endif /* APPALLING_BACKTRACE_HACK */
     if (this_child_connection) connection_delete(this_child_connection);
     if (!post_fork && pidfile)
-            remove_pid_file(pidfile);
-    memset(&sa, 0, sizeof(sa));
+        remove_pid_file(pidfile);
     sa.sa_handler = SIG_DFL;
     sigaction(i, &sa, NULL);
     raise(i);
@@ -140,10 +130,14 @@ void die_signal_handler(const int i) {
 extern int num_running_children; /* in main.c */
 
 #ifdef AUTH_OTHER
-extern pid_t authchild_pid, authchild_died; /* in auth_other.c */
-extern int authchild_wr, authchild_rd, authchild_status;
+extern pid_t auth_other_child_pid, auth_other_childdied; /* in auth_other.c */
+extern int auth_other_childwr, auth_other_childrd, auth_other_childstatus;
 #endif /* AUTH_OTHER */
 
+/* Save information about any child which dies with a signal. */
+pid_t child_died;
+int child_died_signal;
+
 void child_signal_handler(const int i) {
     pid_t pid;
     int e, status;
@@ -154,16 +148,33 @@ void child_signal_handler(const int i) {
     while (1) {
         pid = waitpid(-1, &status, WNOHANG);
         if (pid > 0) {
+#ifdef REALLY_UGLY_PAM_HACK
+            extern auth_pam_child_pid; /* in auth_pam.c */
+            if (pid == auth_pam_child_pid)
+                ; /* Do nothing; in principle we should check to see if it crashed. */
+            else
+#endif /* REALLY_UGLY_PAM_HACK */
 #ifdef AUTH_OTHER
-            if (pid == authchild_pid) {
-                authchild_pid = 0;
-                authchild_died = pid;
-                authchild_status = status;
-                close(authchild_wr);
-                close(authchild_rd);
+            if (pid == auth_other_child_pid) {
+                auth_other_child_pid = 0;
+                auth_other_childdied = pid;
+                auth_other_childstatus = status;
+                close(auth_other_childwr);
+                close(auth_other_childrd);
             } else
 #endif /* AUTH_OTHER */
+            {
                 --num_running_children;
+                /* If the child process was killed by a signal, save its PID
+                 * so that the main daemon can report it. Note that we dont't
+                 * cope with the situation of several children dying nearly
+                 * simultaneously, but this is a `shouldn't happen'
+                 * anyway.... */
+                if (WIFSIGNALED(status)) {
+                    child_died = pid;
+                    child_died_signal = WTERMSIG(status);
+                }
+            }
         } else if (pid == 0 || (pid == -1 && errno != EINTR)) {
             errno = e;
             return;
@@ -173,8 +184,6 @@ void child_signal_handler(const int i) {
 
 /* restart_signal_handler:
  * Signal handler to restart the server on receiving a SIGHUP. */
-extern int restart, post_fork;              /* in main.c */
-
 void restart_signal_handler(const int i) {
     if (!post_fork) {
         foad = i;
