@@ -4,7 +4,7 @@
  *
  * Copyright (c) 2001 Chris Lightfoot. All rights reserved.
  *
- * $Id: util.h,v 1.22 2002/03/18 23:47:34 chris Exp $
+ * $Id: util.h,v 1.26 2003/11/06 01:19:27 chris Exp $
  *
  */
 
@@ -45,6 +45,11 @@ void *myrealloc(char *f, int l, void *p, const size_t n);
 #define realloc(a, b)   myrealloc(__FILE__, __LINE__, a, b)
 #endif
 
+/* alloc_struct S P
+ * Make P point to a new struct S, initialised as if in static storage (like
+ * = {0}). */
+#define alloc_struct(S, p)  do { struct S as__z = {0}; p = xmalloc(sizeof *p); *p = as__z; } while (0)
+
 /* reallocating strncat. */
 char *xstrncat(char *pfx, const char *sfx, const size_t n);
 
@@ -108,10 +113,18 @@ char *xstrdup(const char *s);
 #   define xstrdup  strdup
 #endif /* !MTRACE_DEBUGGING */
 
+char *xstrndup(const char *s, const size_t count);
+
+
+void (*xsignal(int signum, void(*handler)(int)))(int);
+
 /* MD5 digests. */
 void md5_digest(const void *v, const size_t n, unsigned char *md5);
 char *md5_digest_str(const void *v, const size_t n, const int base64);
 
+/* Mapping length. */
+size_t getmaplength(const size_t len);
+
 /* Optional internationalisation support. */
 #ifdef WITH_I18N
 #   include <gettext.h>
